@@ -1,5 +1,6 @@
 /*
-  Copyright(C) 2010-2018 Brazil
+  Copyright (C) 2010-2018  Brazil
+  Copyright (C) 2025  Sutou Kouhei <kou@clear-code.com>
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -25,15 +26,19 @@
 extern "C" {
 #endif
 
-GRN_API char *grn_path_separator_to_system(char *dest, char *groonga_path);
+GRN_API char *
+grn_path_separator_to_system(char *dest, char *groonga_path);
 
-void grn_p_record(grn_ctx *ctx, grn_obj *table, grn_id id);
-grn_rc grn_record_inspect_without_columns(grn_ctx *ctx,
-                                          grn_obj *buffer,
-                                          grn_obj *record);
-grn_rc grn_uvector_record_inspect_without_columns(grn_ctx *ctx,
-                                                  grn_obj *buffer,
-                                                  grn_obj *records);
+void
+grn_p_record(grn_ctx *ctx, grn_obj *table, grn_id id);
+grn_rc
+grn_record_inspect_without_columns(grn_ctx *ctx,
+                                   grn_obj *buffer,
+                                   grn_obj *record);
+grn_rc
+grn_uvector_record_inspect_without_columns(grn_ctx *ctx,
+                                           grn_obj *buffer,
+                                           grn_obj *records);
 
 /*
  * grn_mkstemp generates a unique filename from path_template, creates a
@@ -41,13 +46,21 @@ grn_rc grn_uvector_record_inspect_without_columns(grn_ctx *ctx,
  * The last 6 bytes of path_template must be "XXXXXX" and these are replaced
  * with a string that makes the filename unique.
  */
-int grn_mkstemp(char *path_template);
-bool grn_path_exist(const char *path);
-
-int grn_tokenize(const char *str, size_t str_len,
-                 const char **tokbuf, int buf_size,
-                 const char **rest);
+int
+grn_mkstemp(char *path_template);
+bool
+grn_path_exist(const char *path);
+grn_rc
+grn_path_copy(grn_ctx *ctx,
+              const char *source_path,
+              const char *destination_path);
 
+int
+grn_tokenize(const char *str,
+             size_t str_len,
+             const char **tokbuf,
+             int buf_size,
+             const char **rest);
 
 #ifdef __cplusplus
 }
