@@ -53,6 +53,7 @@ function setup_with_apt () {
         g++
         gcc
         gettext
+        git
         graphviz
         libarrow-compute-dev
         libarrow-dev
@@ -70,6 +71,11 @@ function setup_with_apt () {
         rapidjson-dev
         zlib1g-dev
       )
+      if [ "${distribution}-${code_name}" != "ubuntu-noble" ]; then
+        package_names+=(
+          libfaiss-dev
+        )
+      fi
       ;;
   esac
 
@@ -119,6 +125,10 @@ function setup_with_dnf () {
           ;;
         *)
           ${SUDO} dnf config-manager --set-enabled crb
+          package_names+=(
+            blas-devel
+            lapack-devel
+          )
           ;;
       esac
 
